/*
 * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include <WebCore/ShareableBitmapHandle.h>
#include <wtf/Forward.h>
#include <wtf/RuntimeApplicationChecks.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>

namespace WebCore {
class IntSize;
}

namespace Messages {
namespace AcceleratedBackingStoreDMABuf {

static inline IPC::ReceiverName messageReceiverName()
{
    return IPC::ReceiverName::AcceleratedBackingStoreDMABuf;
}

class DidCreateBuffer {
public:
    using Arguments = std::tuple<uint64_t, WebCore::IntSize, uint32_t, Vector<UnixFileDescriptor>, Vector<uint32_t>, Vector<uint32_t>, uint64_t, WebKit::DMABufRendererBufferFormat::Usage>;

    static IPC::MessageName name() { return IPC::MessageName::AcceleratedBackingStoreDMABuf_DidCreateBuffer; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    DidCreateBuffer(uint64_t id, const WebCore::IntSize& size, uint32_t format, const Vector<UnixFileDescriptor>& fds, const Vector<uint32_t>& offsets, const Vector<uint32_t>& strides, uint64_t modifier, const WebKit::DMABufRendererBufferFormat::Usage& usage)
        : m_id(id)
        , m_size(size)
        , m_format(format)
        , m_fds(fds)
        , m_offsets(offsets)
        , m_strides(strides)
        , m_modifier(modifier)
        , m_usage(usage)
    {
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_id;
        encoder << m_size;
        encoder << m_format;
        encoder << m_fds;
        encoder << m_offsets;
        encoder << m_strides;
        encoder << m_modifier;
        encoder << m_usage;
    }

private:
    uint64_t m_id;
    const WebCore::IntSize& m_size;
    uint32_t m_format;
    const Vector<UnixFileDescriptor>& m_fds;
    const Vector<uint32_t>& m_offsets;
    const Vector<uint32_t>& m_strides;
    uint64_t m_modifier;
    const WebKit::DMABufRendererBufferFormat::Usage& m_usage;
};

class DidCreateBufferSHM {
public:
    using Arguments = std::tuple<uint64_t, WebCore::ShareableBitmapHandle>;

    static IPC::MessageName name() { return IPC::MessageName::AcceleratedBackingStoreDMABuf_DidCreateBufferSHM; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    DidCreateBufferSHM(uint64_t id, WebCore::ShareableBitmapHandle&& handle)
        : m_id(id)
        , m_handle(WTFMove(handle))
    {
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_id;
        encoder << WTFMove(m_handle);
    }

private:
    uint64_t m_id;
    WebCore::ShareableBitmapHandle&& m_handle;
};

class DidDestroyBuffer {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::AcceleratedBackingStoreDMABuf_DidDestroyBuffer; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    explicit DidDestroyBuffer(uint64_t id)
        : m_id(id)
    {
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_id;
    }

private:
    uint64_t m_id;
};

class Frame {
public:
    using Arguments = std::tuple<uint64_t, Vector<WebCore::IntRect, 1>, UnixFileDescriptor>;

    static IPC::MessageName name() { return IPC::MessageName::AcceleratedBackingStoreDMABuf_Frame; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    Frame(uint64_t id, const Vector<WebCore::IntRect, 1>& damage, const UnixFileDescriptor& syncFD)
        : m_id(id)
        , m_damage(damage)
        , m_syncFD(syncFD)
    {
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_id;
        encoder << m_damage;
        encoder << m_syncFD;
    }

private:
    uint64_t m_id;
    const Vector<WebCore::IntRect, 1>& m_damage;
    const UnixFileDescriptor& m_syncFD;
};

} // namespace AcceleratedBackingStoreDMABuf
} // namespace Messages
