/*
 * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(GPU_PROCESS)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include "WebGPUColor.h"
#include <WebCore/WebGPUIndexFormat.h>
#include <WebCore/WebGPUIntegralTypes.h>
#include <wtf/Forward.h>
#include <wtf/RuntimeApplicationChecks.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebKit {
struct WebGPUIdentifierType;
using WebGPUIdentifier = AtomicObjectIdentifier<WebGPUIdentifierType>;
}

namespace Messages {
namespace RemoteRenderPassEncoder {

static inline IPC::ReceiverName messageReceiverName()
{
#if ASSERT_ENABLED
    static std::once_flag onceFlag;
    std::call_once(
        onceFlag,
        [&] {
            ASSERT(isInGPUProcess());
        }
    );
#endif
    return IPC::ReceiverName::RemoteRenderPassEncoder;
}

class Destruct {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_Destruct; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    Destruct()
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
    }

private:
};

class SetPipeline {
public:
    using Arguments = std::tuple<WebKit::WebGPUIdentifier>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetPipeline; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit SetPipeline(const WebKit::WebGPUIdentifier& identifier)
        : m_identifier(identifier)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_identifier;
    }

private:
    const WebKit::WebGPUIdentifier& m_identifier;
};

class SetIndexBuffer {
public:
    using Arguments = std::tuple<WebKit::WebGPUIdentifier, WebCore::WebGPU::IndexFormat, std::optional<WebCore::WebGPU::Size64>, std::optional<WebCore::WebGPU::Size64>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetIndexBuffer; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    SetIndexBuffer(const WebKit::WebGPUIdentifier& identifier, const WebCore::WebGPU::IndexFormat& indexFormat, const std::optional<WebCore::WebGPU::Size64>& offset, const std::optional<WebCore::WebGPU::Size64>& size)
        : m_identifier(identifier)
        , m_indexFormat(indexFormat)
        , m_offset(offset)
        , m_size(size)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_identifier;
        encoder << m_indexFormat;
        encoder << m_offset;
        encoder << m_size;
    }

private:
    const WebKit::WebGPUIdentifier& m_identifier;
    const WebCore::WebGPU::IndexFormat& m_indexFormat;
    const std::optional<WebCore::WebGPU::Size64>& m_offset;
    const std::optional<WebCore::WebGPU::Size64>& m_size;
};

class SetVertexBuffer {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Index32, WebKit::WebGPUIdentifier, std::optional<WebCore::WebGPU::Size64>, std::optional<WebCore::WebGPU::Size64>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetVertexBuffer; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    SetVertexBuffer(const WebCore::WebGPU::Index32& slot, const WebKit::WebGPUIdentifier& identifier, const std::optional<WebCore::WebGPU::Size64>& offset, const std::optional<WebCore::WebGPU::Size64>& size)
        : m_slot(slot)
        , m_identifier(identifier)
        , m_offset(offset)
        , m_size(size)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_slot;
        encoder << m_identifier;
        encoder << m_offset;
        encoder << m_size;
    }

private:
    const WebCore::WebGPU::Index32& m_slot;
    const WebKit::WebGPUIdentifier& m_identifier;
    const std::optional<WebCore::WebGPU::Size64>& m_offset;
    const std::optional<WebCore::WebGPU::Size64>& m_size;
};

class UnsetVertexBuffer {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Index32, std::optional<WebCore::WebGPU::Size64>, std::optional<WebCore::WebGPU::Size64>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_UnsetVertexBuffer; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    UnsetVertexBuffer(const WebCore::WebGPU::Index32& slot, const std::optional<WebCore::WebGPU::Size64>& offset, const std::optional<WebCore::WebGPU::Size64>& size)
        : m_slot(slot)
        , m_offset(offset)
        , m_size(size)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_slot;
        encoder << m_offset;
        encoder << m_size;
    }

private:
    const WebCore::WebGPU::Index32& m_slot;
    const std::optional<WebCore::WebGPU::Size64>& m_offset;
    const std::optional<WebCore::WebGPU::Size64>& m_size;
};

class Draw {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Size32, std::optional<WebCore::WebGPU::Size32>, std::optional<WebCore::WebGPU::Size32>, std::optional<WebCore::WebGPU::Size32>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_Draw; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    Draw(const WebCore::WebGPU::Size32& vertexCount, const std::optional<WebCore::WebGPU::Size32>& instanceCount, const std::optional<WebCore::WebGPU::Size32>& firstVertex, const std::optional<WebCore::WebGPU::Size32>& firstInstance)
        : m_vertexCount(vertexCount)
        , m_instanceCount(instanceCount)
        , m_firstVertex(firstVertex)
        , m_firstInstance(firstInstance)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_vertexCount;
        encoder << m_instanceCount;
        encoder << m_firstVertex;
        encoder << m_firstInstance;
    }

private:
    const WebCore::WebGPU::Size32& m_vertexCount;
    const std::optional<WebCore::WebGPU::Size32>& m_instanceCount;
    const std::optional<WebCore::WebGPU::Size32>& m_firstVertex;
    const std::optional<WebCore::WebGPU::Size32>& m_firstInstance;
};

class DrawIndexed {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Size32, std::optional<WebCore::WebGPU::Size32>, std::optional<WebCore::WebGPU::Size32>, std::optional<WebCore::WebGPU::SignedOffset32>, std::optional<WebCore::WebGPU::Size32>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_DrawIndexed; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    DrawIndexed(const WebCore::WebGPU::Size32& indexCount, const std::optional<WebCore::WebGPU::Size32>& instanceCount, const std::optional<WebCore::WebGPU::Size32>& firstIndex, const std::optional<WebCore::WebGPU::SignedOffset32>& baseVertex, const std::optional<WebCore::WebGPU::Size32>& firstInstance)
        : m_indexCount(indexCount)
        , m_instanceCount(instanceCount)
        , m_firstIndex(firstIndex)
        , m_baseVertex(baseVertex)
        , m_firstInstance(firstInstance)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_indexCount;
        encoder << m_instanceCount;
        encoder << m_firstIndex;
        encoder << m_baseVertex;
        encoder << m_firstInstance;
    }

private:
    const WebCore::WebGPU::Size32& m_indexCount;
    const std::optional<WebCore::WebGPU::Size32>& m_instanceCount;
    const std::optional<WebCore::WebGPU::Size32>& m_firstIndex;
    const std::optional<WebCore::WebGPU::SignedOffset32>& m_baseVertex;
    const std::optional<WebCore::WebGPU::Size32>& m_firstInstance;
};

class DrawIndirect {
public:
    using Arguments = std::tuple<WebKit::WebGPUIdentifier, WebCore::WebGPU::Size64>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_DrawIndirect; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    DrawIndirect(const WebKit::WebGPUIdentifier& indirectBuffer, const WebCore::WebGPU::Size64& indirectOffset)
        : m_indirectBuffer(indirectBuffer)
        , m_indirectOffset(indirectOffset)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_indirectBuffer;
        encoder << m_indirectOffset;
    }

private:
    const WebKit::WebGPUIdentifier& m_indirectBuffer;
    const WebCore::WebGPU::Size64& m_indirectOffset;
};

class DrawIndexedIndirect {
public:
    using Arguments = std::tuple<WebKit::WebGPUIdentifier, WebCore::WebGPU::Size64>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_DrawIndexedIndirect; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    DrawIndexedIndirect(const WebKit::WebGPUIdentifier& indirectBuffer, const WebCore::WebGPU::Size64& indirectOffset)
        : m_indirectBuffer(indirectBuffer)
        , m_indirectOffset(indirectOffset)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_indirectBuffer;
        encoder << m_indirectOffset;
    }

private:
    const WebKit::WebGPUIdentifier& m_indirectBuffer;
    const WebCore::WebGPU::Size64& m_indirectOffset;
};

class SetBindGroup {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Index32, std::optional<WebKit::WebGPUIdentifier>, std::optional<Vector<WebCore::WebGPU::BufferDynamicOffset>>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetBindGroup; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    SetBindGroup(const WebCore::WebGPU::Index32& index, const std::optional<WebKit::WebGPUIdentifier>& identifier, const std::optional<Vector<WebCore::WebGPU::BufferDynamicOffset>>& dynamicOffsets)
        : m_index(index)
        , m_identifier(identifier)
        , m_dynamicOffsets(dynamicOffsets)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_index;
        encoder << m_identifier;
        encoder << m_dynamicOffsets;
    }

private:
    const WebCore::WebGPU::Index32& m_index;
    const std::optional<WebKit::WebGPUIdentifier>& m_identifier;
    const std::optional<Vector<WebCore::WebGPU::BufferDynamicOffset>>& m_dynamicOffsets;
};

class PushDebugGroup {
public:
    using Arguments = std::tuple<String>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_PushDebugGroup; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit PushDebugGroup(const String& groupLabel)
        : m_groupLabel(groupLabel)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_groupLabel;
    }

private:
    const String& m_groupLabel;
};

class PopDebugGroup {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_PopDebugGroup; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    PopDebugGroup()
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
    }

private:
};

class InsertDebugMarker {
public:
    using Arguments = std::tuple<String>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_InsertDebugMarker; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit InsertDebugMarker(const String& markerLabel)
        : m_markerLabel(markerLabel)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_markerLabel;
    }

private:
    const String& m_markerLabel;
};

class SetViewport {
public:
    using Arguments = std::tuple<float, float, float, float, float, float>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetViewport; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    SetViewport(float x, float y, float width, float height, float minDepth, float maxDepth)
        : m_x(x)
        , m_y(y)
        , m_width(width)
        , m_height(height)
        , m_minDepth(minDepth)
        , m_maxDepth(maxDepth)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_x;
        encoder << m_y;
        encoder << m_width;
        encoder << m_height;
        encoder << m_minDepth;
        encoder << m_maxDepth;
    }

private:
    float m_x;
    float m_y;
    float m_width;
    float m_height;
    float m_minDepth;
    float m_maxDepth;
};

class SetScissorRect {
public:
    using Arguments = std::tuple<WebCore::WebGPU::IntegerCoordinate, WebCore::WebGPU::IntegerCoordinate, WebCore::WebGPU::IntegerCoordinate, WebCore::WebGPU::IntegerCoordinate>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetScissorRect; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    SetScissorRect(const WebCore::WebGPU::IntegerCoordinate& x, const WebCore::WebGPU::IntegerCoordinate& y, const WebCore::WebGPU::IntegerCoordinate& width, const WebCore::WebGPU::IntegerCoordinate& height)
        : m_x(x)
        , m_y(y)
        , m_width(width)
        , m_height(height)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_x;
        encoder << m_y;
        encoder << m_width;
        encoder << m_height;
    }

private:
    const WebCore::WebGPU::IntegerCoordinate& m_x;
    const WebCore::WebGPU::IntegerCoordinate& m_y;
    const WebCore::WebGPU::IntegerCoordinate& m_width;
    const WebCore::WebGPU::IntegerCoordinate& m_height;
};

class SetBlendConstant {
public:
    using Arguments = std::tuple<WebKit::WebGPU::Color>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetBlendConstant; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit SetBlendConstant(const WebKit::WebGPU::Color& color)
        : m_color(color)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_color;
    }

private:
    const WebKit::WebGPU::Color& m_color;
};

class SetStencilReference {
public:
    using Arguments = std::tuple<WebCore::WebGPU::StencilValue>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetStencilReference; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit SetStencilReference(const WebCore::WebGPU::StencilValue& stencilValue)
        : m_stencilValue(stencilValue)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_stencilValue;
    }

private:
    const WebCore::WebGPU::StencilValue& m_stencilValue;
};

class BeginOcclusionQuery {
public:
    using Arguments = std::tuple<WebCore::WebGPU::Size32>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_BeginOcclusionQuery; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit BeginOcclusionQuery(const WebCore::WebGPU::Size32& queryIndex)
        : m_queryIndex(queryIndex)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_queryIndex;
    }

private:
    const WebCore::WebGPU::Size32& m_queryIndex;
};

class EndOcclusionQuery {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_EndOcclusionQuery; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    EndOcclusionQuery()
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
    }

private:
};

class ExecuteBundles {
public:
    using Arguments = std::tuple<Vector<WebKit::WebGPUIdentifier>>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_ExecuteBundles; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit ExecuteBundles(const Vector<WebKit::WebGPUIdentifier>& bundles)
        : m_bundles(bundles)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_bundles;
    }

private:
    const Vector<WebKit::WebGPUIdentifier>& m_bundles;
};

class End {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_End; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    End()
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
    }

private:
};

class SetLabel {
public:
    using Arguments = std::tuple<String>;

    static IPC::MessageName name() { return IPC::MessageName::RemoteRenderPassEncoder_SetLabel; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;
    static constexpr bool isStreamEncodable = true;
    static constexpr bool isStreamBatched = false;

    explicit SetLabel(const String& label)
        : m_label(label)
    {
        ASSERT(isInWebProcess());
    }

    template<typename Encoder>
    void encode(Encoder& encoder)
    {
        encoder << m_label;
    }

private:
    const String& m_label;
};

} // namespace RemoteRenderPassEncoder
} // namespace Messages

#endif // ENABLE(GPU_PROCESS)
